@@ -3,34 +3,30 @@
 #
 #' Tuning and unbiased evaluation (single tuning).
 #'
-#' For the first \code{.conf} file in \code{tdm$runList} call the first tuning algorithm
+#' For the first configuration name \code{.conf} in \code{tdm$runList} call the first tuning algorithm
 #' in \code{tdm$tuneMethod} (via function \code{\link{tdmDispatchTuner}}). 
 #' After tuning perform with the best parameters a run of \code{tdm$unbiasedFunc}
 #' (usually \code{\link{unbiasedRun}}). \cr
 #' This experiment is repeated \code{tdm$nExperim} times. 
 #' 
 #Details:
-#' \code{tdmTuneIt} differs from \code{\link{tdmBigLoop}} in that it processes only one \code{.conf} 
-#' file and that it has \code{dataObj} as a mandatory calling parameter. This simplifies the data 
-#' flow and is thus less error-prone.
+#' \code{tdmTuneIt} differs from \code{\link{tdmBigLoop}} in that it processes only one configuration 
+#' \code{.conf} and that it has \code{dataObj} as a mandatory calling parameter. This simplifies  
+#' the data flow and is thus less error-prone.
 #' \cr\cr
 #' \code{tdm} refers to \code{envT$tdm}.
-#' \cr\cr
-#' Tuning is skipped if \code{spotStep=="rep"}. In this case it is assumed then that 
-#' \code{envT$bstGrid} and \code{envT$resGrid} contain the appropriate data frames already.
-#' See \code{\link{tdmEnvTAddBstRes}} on how to fill \code{envT$bstGrid} and \code{envT$resGrid}  from an \code{.RData} file.
 #'
 #' See Details in \code{\link{tdmBigLoop}} for the list of avaialble tuners.
 #'
 #' @param envT      an environment containing on input at least the element \code{tdm} (a list with general settings for TDMR, 
 #'                   see \code{\link{tdmDefaultsFill}}), which has at least the elements  
 #'     \describe{
-#'     \item{\code{tdm$runList}}{ \code{.conf} filename }
+#'     \item{\code{tdm$runList}}{ list of configuration names \code{.conf}  }
 #'     \item{\code{tdm$tuneMethod}}{ the tuner }
 #'     }
-#' @param spotStep  \code{["auto"]} which step of SPOT to execute (either \code{"auto"} or \code{"rep"}).
+# @param spotStep  \code{["auto"]} which step of SPOT to execute (either \code{"auto"} or \code{"rep"}).
 #' @param dataObj   object of class \code{\link{TDMdata}} (constructed here with the help of 
-#'      \code{\link{tdmSplitTestData}}).
+#'      \code{\link{tdmReadAndSplit}}).
 #'
 #' @return environment \code{envT}, containing  the results
 #'      \item{res}{ data frame with results from last tuning (one line for each call of \code{tdmStart*})} 
@@ -67,21 +63,23 @@
 #'   \code{tdmBigLoop} and its subfunctions.
 #'
 #' @note Side effects:
-#'   Irrespective of the value of \code{tdm$fileMode}, 
+#   Irrespective of the value of \code{tdm$fileMode}, 
 #'     \itemize{
-#'         \item a compressed version of \code{envT} is saved to file \code{tdm$filenameEnvT} (default: \code{<runList[1]>.RData}), 
-#'               relative to  the directory of the \code{.conf} file. 
+#'         \item a compressed version of \code{envT} is saved to file \code{tdm$filenameEnvT} 
+#'               (default: \code{<runList[1]>.RData}), in current directoy.
 #'     }
 #'   If \code{tdm$U.saveModel==TRUE}, then \code{envT$result$lastRes$lastModel} (the last trained model) will be saved to \code{tdm$filenameEnvT}. 
 #'   The default is \code{tdm$U.saveModel==TRUE} (with \code{tdm$U.saveModel==FALSE} smaller \code{.RData} files).
 #' 
-#'   If \code{tdm$fileMode==TRUE}, more files are written relative to  the directory of the \code{.conf} file:
-#'     \itemize{
-#'         \item \code{envT$theFinals } is written to file \code{tdm$finalFile} 
-#'         \item \code{envT$theFinals } is appended to \code{tdm$experFile}
-#'      }                                                                                                           
-#'   If \code{tdm$finalFile==NULL}, then it is set to \code{sub(".conf",".fin",runList[1])}.  \cr
-#'   If \code{tdm$experFile==NULL}, then nothing is appended to any experiment file.
+#---- now deleted, we act always as if tdm$fileMode==FALSE ---------------- 
+#   If \code{tdm$fileMode==TRUE}, more files are written relative to  the directory of the \code{.conf} file:
+#     \itemize{
+#         \item \code{envT$theFinals } is written to file \code{tdm$finalFile} 
+#         \item \code{envT$theFinals } is appended to \code{tdm$experFile}
+#      }                                                                                                           
+#   If \code{tdm$finalFile==NULL}, then it is set to \code{sub(".conf",".fin",runList[1])}.  \cr
+#   If \code{tdm$experFile==NULL}, then nothing is appended to any experiment file.
+#--------------------------------------------------------------------------
 #'
 #' Example usages of function \code{tdmBigLoop} are shown in \cr
 #' \tabular{ll}{
@@ -103,36 +101,43 @@
 #' #*# Do not expect good numeric results. 
 #' #*# See demo/demo03sonar_B.r for a somewhat longer tuning run, with two tuners SPOT and LHD.
 #' 
-#' ## set working directory (dir with .apd, .conf and main_*.r file)
-#' path <- paste(find.package("TDMR"), "demo02sonar",sep="/");
-#' source(paste(path,"main_sonar.r",sep="/"));    
+#'   ## path is the dir with data and main_*.r file:
+#'   path <- paste(find.package("TDMR"), "demo02sonar",sep="/");
+#'   #path <- paste("../../inst", "demo02sonar",sep="/");
 #' 
-#' ## control settings for TDMR
-#' tdm <- list( mainFunc="main_sonar"
-#'            , runList = c("sonar_04.conf")
-#'            , umode="CV"              # { "CV" | "RSUB" | "TST" | "SP_T" }
-#'            , tuneMethod = c("lhd")
-#'            , filenameEnvT="exBigLoop.RData"   # file to save environment envT (in dir 'path')
-#'            , nrun=1, nfold=2         # repeats and CV-folds for the unbiased runs
-#'            , nExperim=1
-#'            , parallelCPUs=1
-#'            , parallelFuncs=c("readCmdSonar")
-#'            , optsVerbosity = 0       # the verbosity for the unbiased runs
-#'            );
-#' ## Each element of tdm$runList has the settings for one tuning process (e.g. 
-#' ##    - auto.loop.steps = number of SPOT generations       
-#' ##    - auto.loop.evals = budget of model building runs and 
-#' ##    - io.roiFileName = "sonar_04.roi"
-#' ## ). 
-#' 
-#' spotStep = "auto";   
-#' source(paste(path,"start_tuneIt.r",sep="/"),chdir=TRUE);    # change dir to 'path' while sourcing
+#'   ## control settings for TDMR
+#'   tdm <- list( mainFunc="main_sonar"
+#'              , umode="CV"              # { "CV" | "RSUB" | "TST" | "SP_T" }
+#'              , tuneMethod = c("lhd")
+#'              , filenameEnvT="exBigLoop.RData"   # file to save environment envT
+#'              , nrun=1, nfold=2         # repeats and CV-folds for the unbiased runs
+#'              , nExperim=1
+#'              , optsVerbosity = 0       # the verbosity for the unbiased runs
+#'              );
+#'   source(paste(path,"main_sonar.r",sep="/"));    # main_sonar, readTrnSonar
+#' \dontshow{
+#'   #*# This demo is for "R CMD check" (runs in half the time) 
+#'   source(paste(path,"control_sonar_check.r",sep="/")); # controlDM, controlSC
+#' }
+#' \donttest{
+#'   #*# This demo is for example and help (more meaningful, a bit higher budget)
+#'   source(paste(path,"control_sonar.r",sep="/"));       # controlDM, controlSC
+#' }
+#'   ctrlSC <- controlSC();
+#'   ctrlSC$opts <- controlDM();
+#'
+#'   #
+#'   # perform a complete tuning + unbiased eval
+#'   # 
+#'   envT <- tdmEnvTMakeNew(tdm,sCList=list(ctrlSC)); # construct envT from settings in tdm and ctrlSC
+#'   dataObj <- tdmReadTaskData(envT,envT$tdm);
+#'   envT <- tdmTuneIt(envT,dataObj=dataObj);       # start the tuning loop 
 #'
 #' @seealso   \code{\link{tdmBigLoop}}, \code{\link{tdmDispatchTuner}}, \code{\link{unbiasedRun}}
-#' @author Wolfgang Konen (\email{wolfgang.konen@@th-koeln.de})
+#' @author Wolfgang Konen (\email{wolfgang.konen@@th-koeln.de}), THK
 #' @export
 ######################################################################################
-tdmTuneIt <- function(envT,spotStep="auto",dataObj) {
+tdmTuneIt <- function(envT,dataObj) {
                             # The environment envT is passed by reference into the inner functions
                             # which means that it can be used a) to transport information back from
                             # those inner functions and b) to transport information to and back even 
@@ -141,7 +146,8 @@ tdmTuneIt <- function(envT,spotStep="auto",dataObj) {
   
   tdm <- envT$tdm;
   tdm <- tdmMapDesLoad(tdm);
-
+  spotStep="auto";   # --- deprecated, only to have a value for spotStep (no longer used)
+  
   if (is.null(envT$getBst)) envT <- tdmEnvTAddGetters(envT);
   # just in case the user loaded envT via load("myFile.Rdata") and not tdmEnvTLoad("myFile.Rdata") 
       
@@ -160,7 +166,7 @@ tdmTuneIt <- function(envT,spotStep="auto",dataObj) {
 	if (length(indVec)==1 & tdm$parallelCPUs>1) {
 	  warning("There is only one job (length(indVec)==1) --> parallelization would not run, so we set tdm$ParallelCPUs=1");
 	  tdm$parallelCPUs=1;
-  }
+	}
 
   if (tdm$parallelCPUs>1) {
     cl = prepareParallelExec(tdm);
@@ -196,7 +202,7 @@ tdmTuneIt <- function(envT,spotStep="auto",dataObj) {
         nConf = which(confFile==envT$runList);
         
         print(c(ind,confFile,nExp,theTuner));
-        envT$spotConfig <- sC <- envT$sCList[[nConf]]; # spotGetOptions(srcPath=tdm$theSpotPath,confFile);
+        envT$spotConfig <- sC <- envT$sCList[[nConf]]; 
         envT$theTuner <- theTuner;
         envT$nExp <- envT$spotConfig$nExp <- nExp;
         envT$bst <- NULL;
@@ -206,12 +212,13 @@ tdmTuneIt <- function(envT,spotStep="auto",dataObj) {
           envT$res = envT$getRes(envT,confFile,nExp,theTuner);
         }
 
-        if (tdm$fileMode) {  
-          if (is.null(tdm$finalFile)) tdm$finalFile = sub(".conf",".fin",confFile);
-          tFinalFile <- ifelse(tdm$nExperim>1, sprintf("%s-e%02d%s",sub(".fin","",tdm$finalFile,fixed=TRUE),nExp,".fin"), tdm$finalFile);
-          # i.e. if tdm$finalFile="cpu.fin", then tFinalFile="cpu-e02.fin" for nExp=2
-          if (file.exists(tFinalFile)) file.remove(tFinalFile);
-        } 
+        # if (tdm$fileMode) {  
+        #   if (is.null(tdm$finalFile)) tdm$finalFile = sub(".conf",".fin",confFile);
+        #   tFinalFile <- ifelse(tdm$nExperim>1, sprintf("%s-e%02d%s",sub(".fin","",tdm$finalFile,fixed=TRUE),nExp,".fin"), tdm$finalFile);
+        #   # i.e. if tdm$finalFile="cpu.fin", then tFinalFile="cpu-e02.fin" for nExp=2
+        #   if (file.exists(tFinalFile)) file.remove(tFinalFile);
+        # }
+        
         # NEW 09/2012: always operate SPOT with spot.fileMode=FALSE (take everything from  envT$spotConfig)
         envT$spotConfig$spot.fileMode=FALSE;
         
@@ -224,7 +231,7 @@ tdmTuneIt <- function(envT,spotStep="auto",dataObj) {
             # this is for the case spotStep=="rep":
             envT$spotConfig$alg.currentResult <- envT$res;
             envT$spotConfig$alg.currentBest <- envT$bst;	
-            writeLines(sprintf("*** Starting TUNER %s, spotStep=%s, on task %s ***\n",theTuner,spotStep,confFile),con=stderr());
+            writeLines(sprintf("\n*** Starting TUNER %s on task %s, experiment %d ***",theTuner,confFile,nExp),con=stderr());
             tdmDispatchTuner(theTuner,confFile,spotStep,tdm,envT,dataObj);
             # If spotStep=="auto" then tdmDispatchTuner runs the tuning process, puts its results in envT$bst, envT$res 
             # and returns the (extended) spotConfig in envT$tunerVal.
@@ -243,62 +250,64 @@ tdmTuneIt <- function(envT,spotStep="auto",dataObj) {
         cat1(opts2,paste(time.txt[tdm$timeMode], "time for tuning with tdmDispatchTuner:",envT$time.TRN,"sec\n"));     
 
         ptm <- proc.time();
-        finals <- NULL;
+        envT$finals <- NULL;
         if (tdm$nrun>0) {
-          writeLines(paste("*** starting",tdm$unbiasedFunc,"for",confFile,"with umode=",tdm$umode,"***\n"),con=stderr());
-          cmd=paste("finals <-",tdm$unbiasedFunc,"(confFile,envT,dataObj,umode=tdm$umode,finals=finals,withParams=envT$wP,tdm=tdm)",sep="");
-          eval(parse(text=cmd));
+          writeLines(sprintf("\n*** Starting %s for %s with umode = %s ***",tdm$unbiasedFunc,confFile,tdm$umode),con=stderr());
+          # cmd=paste("envT <-",tdm$unbiasedFunc,"(confFile,envT,dataObj,umode=tdm$umode,withParams=envT$wP,tdm=tdm)",sep="");
+          # eval(parse(text=cmd));
+          envT <- eval(call(tdm$unbiasedFunc,confFile,envT,dataObj,umode=tdm$umode,withParams=envT$wP,tdm=tdm))
           time.TST=(proc.time()-ptm)[tdm$timeMode];   
           cat1(opts2,paste(time.txt[tdm$timeMode], "time for",tdm$unbiasedFunc,":",time.TST,"sec\n"));   
-          finals <- cbind(finals,Time.TST=time.TST);  
+          envT$finals <- cbind(envT$finals,Time.TST=time.TST);  
 
           # transport back opts$srf (data frame with SRF info in case opts$SRF.calc==TRUE)
           envT$sCList[[nConf]]$opts$srf = envT$result$lastRes$opts$srf
 
-          if (tdm$fileMode) {  
-            #removeTmpfiles2(confFile);
-            if (!file.exists(dirname(tFinalFile))) {
-              success = dir.create(dirname(tFinalFile));     
-              if (!success) stop(sprintf("Could not create dirname(tFinalFile)=%s",dirname(tFinalFile)));
-            }
-        		colNames = ifelse(file.exists(tFinalFile),FALSE,TRUE);
-        		write.table(finals
-        				, file = tFinalFile
-        				, col.names= colNames
-        				, row.names= FALSE
-        				, append = !colNames
-        				, sep = " ",
-        				, quote = FALSE
-        				, eol = "\n"
-        		);
-        		if (!is.null(tdm$experFile)) {
-              if (!file.exists(dirname(tdm$experFile))) {
-                success = dir.create(dirname(tdm$experFile));     
-                if (!success) stop(sprintf("Could not create dirname(tdm$experFile)=%s",dirname(tdm$experFile)));
-              }
-          		colNames = ifelse(file.exists(tdm$experFile),FALSE,TRUE);
-          		write.table(finals             # multiple execution of same experiment (nExperim>1 in script_all.R)
-          				, file = tdm$experFile
-          				, col.names= colNames
-          				, row.names= FALSE
-          				, append = !colNames
-          				, sep = " ",
-          				, quote = FALSE
-          				, eol = "\n"
-          		);
-        		}
-          } # if(tdm$fileMode)
+        #   if (tdm$fileMode) {  
+        #     #removeTmpfiles2(confFile);
+        #     if (!file.exists(dirname(tFinalFile))) {
+        #       success = dir.create(dirname(tFinalFile));     
+        #       if (!success) stop(sprintf("Could not create dirname(tFinalFile)=%s",dirname(tFinalFile)));
+        #     }
+        # 		colNames = ifelse(file.exists(tFinalFile),FALSE,TRUE);
+        # 		write.table(envT$finals
+        # 				, file = tFinalFile
+        # 				, col.names= colNames
+        # 				, row.names= FALSE
+        # 				, append = !colNames
+        # 				, sep = " ",
+        # 				, quote = FALSE
+        # 				, eol = "\n"
+        # 		);
+        # 		if (!is.null(tdm$experFile)) {
+        #       if (!file.exists(dirname(tdm$experFile))) {
+        #         success = dir.create(dirname(tdm$experFile));     
+        #         if (!success) stop(sprintf("Could not create dirname(tdm$experFile)=%s",dirname(tdm$experFile)));
+        #       }
+        #   		colNames = ifelse(file.exists(tdm$experFile),FALSE,TRUE);
+        #   		write.table(envT$finals       # multiple execution of same experiment (nExperim>1 in script_all.R)
+        #   				, file = tdm$experFile
+        #   				, col.names= colNames
+        #   				, row.names= FALSE
+        #   				, append = !colNames
+        #   				, sep = " ",
+        #   				, quote = FALSE
+        #   				, eol = "\n"
+        #   		);
+        # 		}
+        #   } # if(tdm$fileMode)
           flush.console(); 
         } # if(tdm$nrun>0)
          
-        # we return a *list* with the most important elements from environment envT and not envT itself, because 
-        # parSapply (parallel execution) can not deal with environments as return values.
+        # we return a *list* sappResult with the most important elements from environment envT and 
+        # not envT itself, because parSapply (parallel execution) can not deal with environments as 
+        # return values.
         list( tunerVal=envT$tunerVal    # last tuning result: spotConfig with extensions       
              ,bst=envT$bst              # last tuning result       
              ,res=envT$res              # last tuning result  
              ,result=envT$result        # last tuning result
              ,roi=envT$tunerVal$alg.roi      
-             ,theFinals=finals   ###tail(envT$theFinals,1)
+             ,theFinals=envT$finals   
              );      
   		} # end of function tuneItStep
   		#------------------------------------------------------------------------------------------------------
